<?php
class WP_KeepUp_admin {
    public function __construct() {
        // Add admin menu
        add_action('admin_menu', array($this, 'wp_keep_up_admin_menu'));

        // Register security settings
        add_action('admin_init', array($this, 'register_security_settings'));
        
    }

public function wp_keep_up_admin_menu() {
    add_menu_page(
        __('WP KeepSoul', 'wp-keep-up'),
        __('WP KeepSoul', 'wp-keep-up'),
        'manage_options',
        'wp-keep-up',
        array($this, 'wp_keep_up_admin_page'),
        'dashicons-shield-alt',
        2
    );

    // Add custom styles for the admin menu
    add_action('admin_head', array($this, 'custom_admin_menu_styles'));
}

public function custom_admin_menu_styles() {
    echo '<style>
        #toplevel_page_wp-keep-up .wp-menu-name {
            font-weight: bold; /* Bold text */
            color: white !important; /* White text color */
        }
        #toplevel_page_wp-keep-up {
            background-color: red !important; /* Red background for the menu item */
        }
    </style>';
}


    public function register_security_settings() {
        register_setting('wp_keepsoul_admin_settings', 'wp_keepsoul_admin_security');
    }

    public function wp_keep_up_admin_page() {
        // 1. Process RESTORE if the user clicked 'Restore Latest Backup'
        if ( isset($_POST['restore_functions_backup']) 
             && check_admin_referer('wp_keep_up_restore_nonce_action', 'wp_keep_up_restore_nonce_field')
           ) 
        {
            $restore_result = $this->wp_keep_up_restore_functions_php();
            if ( $restore_result === true ) {
                echo '<div class="notice notice-success"><p><strong>functions.php</strong> was restored successfully!</p></div>';
            } else {
                echo '<div class="notice notice-error"><p>Unable to restore the backup. ' . esc_html($restore_result) . '</p></div>';
            }
        }

        // 2. If a scan is requested, handle it before rendering the UI
        if ( isset($_GET['scan']) ) {
            $scan_type = sanitize_text_field( $_GET['scan'] );

            switch ($scan_type) {
                case 'admin-panel-protect':
                    // admin-panel-protect.php
                    require_once WP_PLUGIN_DIR . '/wp-keep-up/admin-panel-protect.php';
                    break;

                case 'security-header':
                    // security-header.php
                    require_once WP_PLUGIN_DIR . '/wp-keep-up/security-header.php';
                    break;
            }
        }

        // 3. Page title
        $title    = get_admin_page_title();
        $file_url = home_url('wp-content/wp-keep-up/log.csv');
        $action   = wp_create_nonce('wp-keepUp-nonce-action');

        // 4. Get the security option
        $security_option = get_option('wp_keepsoul_admin_security', 'off');
        $security_status = $security_option === 'on' ? 'Enabled' : 'Disabled';

        // 5. Render the main admin page content
        echo <<<HTML
        <div class="wrap">
            <h1 style="margin-bottom: 1rem;">{$title}</h1>
            
<style>
    .wp-keep-up-note {
        background: linear-gradient(to right, #fff8e1, #ffecb3); /* Modern gradient background */
        border: 1px solid #ffd54f; /* Softer yellow border */
        padding: 1.2rem;
        border-radius: 8px; /* Slightly more rounded corners */
        margin-bottom: 2rem;
        max-width: 800px;
        box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1); /* Soft shadow */
    }
    .wp-keep-up-note strong {
        font-weight: 700;
        color: #ff6f00; /* Highlighted text color */
    }
    .wp-keep-up-scans {
        display: flex;
        flex-wrap: wrap;
        gap: 2rem;
        max-width: 1000px;
        justify-content: space-between; /* Better alignment */
    }
    .wp-keep-up-scan-box {
        flex: 1;
        min-width: 300px;
        background: linear-gradient(to bottom, #ffffff, #f0f0f5); /* Subtle glossy gradient */
        border: 1px solid #ddd;
        border-radius: 12px; /* More rounded corners */
        padding: 1.5rem;
        box-shadow: 0 4px 10px rgba(0, 0, 0, 0.1); /* Enhanced shadow */
        transition: transform 0.3s ease, box-shadow 0.3s ease; /* Smooth hover effect */
    }
    .wp-keep-up-scan-box:hover {
        transform: translateY(-5px); /* Lift on hover */
        box-shadow: 0 8px 20px rgba(0, 0, 0, 0.15); /* Stronger shadow */
    }
    .wp-keep-up-scan-box h2 {
        margin-top: 0;
        margin-bottom: 0.5rem;
        font-size: 1.5rem;
        color: #333;
        border-bottom: 2px solid #4CAF50; /* Stylish underline */
        padding-bottom: 0.3rem;
    }
    .wp-keep-up-scan-box ul {
        margin-top: 0.5rem;
        padding-left: 1.5rem;
        list-style: disc;
        color: #555;
    }
    .wp-keep-up-scan-box p {
        margin: 0.5rem 0;
        color: #666;
        font-size: 0.95rem;
    }
    .wp-keep-up-radio {
        margin-top: 1rem;
    }
    .wp-keep-up-radio label {
        cursor: pointer;
        font-size: 1rem;
        color: #333;
    }
    .wp-keep-up-start-scan {
        margin-top: 2rem;
        text-align: left; /* Center-align buttons */
    }
    .glossy-button {
        display: inline-block;
        background: linear-gradient(to bottom, #4CAF50, #3E8E41); /* Glossy green button */
        color: #fff;
        padding: 12px 24px;
        text-decoration: none;
        border-radius: 6px; /* Slightly more rounded corners */
        border: 1px solid #388e3c; /* Button border */
        cursor: pointer;
        font-weight: 600;
        font-size: 1rem;
        transition: all 0.3s ease; /* Smooth hover effect */
        box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1); /* Subtle shadow */
    }
    .glossy-button:hover {
        background: linear-gradient(to bottom, #45a049, #3e8e41); /* Slightly brighter on hover */
        box-shadow: 0 6px 12px rgba(0, 0, 0, 0.2); /* Enhanced shadow on hover */
        transform: scale(1.05); /* Slight zoom on hover */
    }
    .wp-keep-up-instructions p {
        margin: 0.6rem 0; /* Adjusted spacing */
        font-size: 1rem;
        color: #444;
        line-height: 1.6; /* Improved readability */
    }
</style>

            <img src="https://raj-bhat.com/wp-content/uploads/2025/01/wp-keepsoul-2.O.jpg" alt="WP-KeepSoul 2.O Logo"width="250" height="250">
          <h2 style="font-size: 24px;"><strong>WP KeepSoul 2.0</strong></h2>
            <img src="https://raj-bhat.com/wp-content/uploads/2025/01/cropped-raj-b-logo@2x.png" alt="Logo"width="150" height="150">
            <h2 style="font-size: 24px;"><strong>RAJ BHAT - Orbem Enterprise</strong></h2>
            <div class="wp-keep-up-note">
                <p style="color:#f00; font-weight:600; font-size:1rem;">
                    <strong>Note:</strong> READ INSTRUCTION CAREFULLY! POWERFUL TOOL !!
                </p>
                <p>
                    If your WordPress site is custom-developed and has additional support to run in the root directory,
                    like (.htaccess with specific rules) (WP-CONTENT EXCLUDED), 
                    PLEASE MOVE THOSE TO THE FOLLOWING DIRECTORY!
                </p>
                <p style="color:#f00; font-weight:400; font-size:1rem;">
                    <strong>/wp-content/wp-keep-up/backup</strong>
                </p>
                <p>THESE FILES WILL NOT BE SCANNED AND REPLACED EVERY TIME FROM THE BACKUP DIRECTORY ITSELF.</p>
                <p>Keep your WordPress site free of file injection attacks and misconfigurations.</p>
            </div>

            <div class="wp-keep-up-instructions">
                A log file will be generated and mailed to the administrator's email address.
                    You can also use "Download Log" and press CTRL+S on the next page to save a copy of the log.
                </p>
            </div>

            <form method="GET" action="">
                <!-- Add necessary hidden fields for the scan action -->
                <input type="hidden" name="wp-keepUp-nonce" value="{$action}" />
                <input type="hidden" name="page" value="wp-keep-up" />

                <div class="wp-keep-up-scans">
                    <!-- Quick Scan Box -->
                    <div class="wp-keep-up-scan-box">
                        <h2>Quick Scan</h2>
                        <p style="color:#d95707; font-weight:600;">Fast (~5 seconds)</p>
                        <p>
                            Detects <strong>existing malware</strong> but may produce false negatives. 
                            It focuses on:
                        </p>
                        <ul>
                            <li>Malicious code appended to existing files</li>
                            <li>File permission recording</li>
                        </ul>
                        <p>
                            Only scans a few core files. For a thorough check, consider Full Scan.
                        </p>
                        <div class="wp-keep-up-radio">
                            <label>
                                <input type="radio" name="scan" value="quick-scan">
                                Select Quick Scan
                            </label>
                        </div>
                    </div>

                    <!-- Full Scan Box -->
                    <div class="wp-keep-up-scan-box">
                        <h2>Full Scan</h2>
                        <p style="color:#008f0a; font-weight:600;">Comprehensive (~1 minute)</p>
                        <p>
                            Performs a deep check, including malicious file uploads, 
                            missing core file detection, malicious code injection, etc.
                        </p>
                        <ul>
                            <li>Detect and delete unwanted/malicious files</li>
                            <li>Checks for missing core files</li>
                            <li>Checks file permissions</li>
                        </ul>
                        <p>
                            Version-independent scan — supports upcoming versions without extra updates.
                        </p>
                        <div class="wp-keep-up-radio">
                            <label>
                                <input type="radio" name="scan" value="full-scan">
                                Select Full Scan
                            </label>
                        </div>
                    </div>

                    <!-- Admin Panel Protect Box -->
                    <div class="wp-keep-up-scan-box">
                        <h2>Admin Panel Protect</h2>
                        <p style="color: #1d70b8; font-weight:600;">Additional Hardening</p>
                        <p>
                            This runs <strong>admin-panel-protect.php</strong> with extra security measures. 
                            e.g., disable file editing in WP admin, force HTTPS, etc.
                        </p>
                        <div class="wp-keep-up-radio">
                            <label>
                                <input type="radio" name="scan" value="admin-panel-protect">
                                Run Admin Panel Protect
                            </label>
                        </div>
                    </div>

                    <!-- Security Header Box -->
                    <div class="wp-keep-up-scan-box">
                        <h2>Security Header</h2>
                        <p style="color: #1d70b8; font-weight:600;">HTTP Header Protection</p>
                        <p>
                            Runs <strong>security-header.php</strong> to apply headers like
                            Content-Security-Policy, HSTS, Referrer-Policy, and more.
                        </p>
                        <div class="wp-keep-up-radio">
                            <label>
                                <input type="radio" name="scan" value="security-header">
                                Run Security Header
                            </label>
                        </div>
                    </div>
                </div>

                <!-- Start Button -->
                <div class="wp-keep-up-start-scan">
                    <br>
                    <button type="submit" class="glossy-button">Start</button>
                </div>
            </form>

        <!-- Download Log Section -->
        <div style="margin-top: 2rem; text-align: left;">
            <a class="modern-button" href="{$file_url}">
                Download Scan Logs
            </a>
        </div>

        <!-- Restore Section -->
        <hr style="margin: 2rem 0; border: 0; border-top: 1px solid #ddd;">
        <div style="text-align: left;">
            <h2 style="color: #4CAF50; font-weight: bold; font-size: 1.5rem; margin-bottom: 1rem;">
                Restore functions.php from Backup
            </h2>
            <p style="color: #555; font-size: 1rem; margin-bottom: 1rem;">
                Use this option to restore the most recent backup of <strong>functions.php</strong> 
                if there are issues with your website after using security headers.
            </p>
            <form method="POST" action="">
HTML;
            // WordPress nonce for security on restore
            wp_nonce_field('wp_keep_up_restore_nonce_action', 'wp_keep_up_restore_nonce_field');
            echo <<<HTML
                <button type="submit" class="modern-button modern-button-primary" 
                        name="restore_functions_backup" value="1">
                    Reset Security Headers
                </button>
            </form>
        </div>
    </div>

    <!-- CSS for Modernized Style -->
    <style>
        .modern-button {
            display: inline-block;
            padding: 12px 20px;
            font-size: 1rem;
            font-weight: bold;
            text-align: left;
            color: white;
            text-decoration: none;
            background: linear-gradient(to bottom, #3a8dd5, #236ba9);
            border: none;
            border-radius: 6px;
            box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);
            transition: background 0.3s ease, transform 0.2s ease;
            cursor: pointer;
        }
        .modern-button:hover {
            background: linear-gradient(to bottom, #469ae6, #2b80bf);
            transform: scale(1.05);
        }

        .modern-button-primary {
            background: linear-gradient(to bottom, #4CAF50, #388E3C);
        }
        .modern-button-primary:hover {
            background: linear-gradient(to bottom, #66BB6A, #43A047);
        }

        hr {
            margin: 2rem 0;
        }

        h2 {
            font-family: "Segoe UI", Tahoma, Geneva, Verdana, sans-serif;
        }

        p {
            font-family: "Arial", sans-serif;
        }
    </style>
HTML;
}

    /**
     * Copy the MOST RECENT backup of functions.php to the active theme.
     * Looks for backup files named like "functions-backup-*.php"
     * in wp-content/wp-keep-up/backup/.
     */
    private function wp_keep_up_restore_functions_php() {
        // Active theme's functions.php
        $theme_directory  = get_template_directory();
        $destination_file = $theme_directory . '/functions.php';

        // Backup directory
        $backup_dir = ABSPATH . 'wp-content/wp-keep-up/backup/';

        // Find all possible backup files
        $backup_files = glob($backup_dir . 'functions-backup-*.php');
        if (!$backup_files) {
            return 'No backup files found in: ' . $backup_dir;
        }

        // Sort by modification time, descending (newest first)
        usort($backup_files, function($a, $b) {
            return filemtime($b) - filemtime($a);
        });

        // The most recent backup
        $latest_backup_file = $backup_files[0];

        // Double-check it exists
        if (!file_exists($latest_backup_file)) {
            return 'Latest backup file not found or inaccessible.';
        }

        // Attempt to copy
        if (!@copy($latest_backup_file, $destination_file)) {
            return 'Could not restore the backup file to ' . $destination_file;
        }

        // Success!
        return true;
    }
    
    
}
