<?php
class WP_KeepUp_Full_San{

public function run() {
    $start_time = microtime(true); // time switched on for micro time counter

    // Download WordPress
    $this->download_wordpress(); // downloads wordpress everytime when full scan is selected
    // Log file
    $log_file = WP_KEEP_UP_RESOURCE .'log.csv'; // accessing log file to maintain logs

    // Will exclude everything under these directories
    $exclude_dir = array('lib', 'wp-content'); // purposly excluded for this project
    $exclude_file = array('inject.php');// can add exclude in future if any files to be excluded purposly excluded for this project
    $filter = function ($file, $key, $iterator) use ($exclude_dir) {
        if ($iterator->hasChildren() && !in_array($file->getFilename(), $exclude_dir)) {
            return true;
        }
        return $file->isFile();
    };
    
    // WP root directory
    $innerIterator = new RecursiveDirectoryIterator(
        ABSPATH,
        RecursiveDirectoryIterator::SKIP_DOTS
    );
    $iterator = new RecursiveIteratorIterator(
        new RecursiveCallbackFilterIterator($innerIterator, $filter)
    );
    /// checking for all files and exclude from filter function above. 
    foreach ($iterator as $pathname => $fileInfo) {
        $log_data = array(date('Y-m-d'), date('H:i:s'), 'Full Scan', strlen(file_get_contents($pathname)));
        

        if(in_array($fileInfo->getFilename(), $exclude_file)) {
            continue;
        }

        // Trim file path to get relative path
        $trim_path = substr($pathname, strlen(ABSPATH));

        // Join the path with the wp source directory
        $wp_source = WP_KEEP_UP_RESOURCE . 'wordpress/' .$trim_path;

        // Join the path with backup directory
        $wp_backup = WP_KEEP_UP_RESOURCE .'backup/' .$trim_path;
        if(!file_exists($wp_backup)) {
            $wp_backup = '';
        }
        
        // quarantine directory, store changed directory
        $quarantine_dir = WP_KEEP_UP_RESOURCE .'quarantine/' .$trim_path;

        // Check file exist in source directory - check if file is legit and 
        if(file_exists( $wp_source )) {
            array_push($log_data, strlen(file_get_contents($wp_source)));
            $this->wp_keep_update($pathname, $wp_source, $wp_backup, $quarantine_dir);

        } else {

            if(!is_dir(dirname($quarantine_dir))) {
                mkdir(dirname($quarantine_dir), 0777, true);
            } 
            copy($pathname, $quarantine_dir);
            if($wp_backup != '') {

                $backup = file_get_contents($wp_backup);
                $extra = file_get_contents($pathname);
                $extra_len = strlen($extra);
                if($backup != $extra) {
                    $backup_len = strlen($backup);
                    file_put_contents($pathname, $backup);
                    array_push($log_data, $backup_len);

                    clearstatcache();
                    $file_permission = substr(sprintf('%o', fileperms($pathname)), -4);
                    array_push($log_data, basename($pathname), $pathname, $file_permission, 'Replaced from backup');
                    $file = fopen($log_file, 'a');
                    fputcsv($file, $log_data);
                    fclose($file);
                }
            } else {
                array_push($log_data, 'No backup');
                clearstatcache();
                $file_permission = substr(sprintf('%o', fileperms($pathname)), -4);
                array_push($log_data, basename($pathname), $quarantine_dir, $file_permission, 'Removed');
                $file = fopen($log_file, 'a');
                fputcsv($file, $log_data);
                fclose($file);

                unlink($pathname);
            }
        }  
    }

    $log_file = WP_KEEP_UP_RESOURCE .'log.csv';

    if($this->dir_is_empty(WP_KEEP_UP_RESOURCE .'quarantine/')) {
    
        $end_time = microtime(true);
        $duration = $end_time - $start_time;
        $this->$duration = $duration;
        $message = '<p>Full Scan completed in '. $duration .' seconds</p><p style="color:#0F0;">Not found any quarantine file</p>';
        wp_keepUp_sendMail($log_file, $message);

    } else {

        $zip_file = WP_KEEP_UP_RESOURCE . quarantine_resource_zip();
        $attachment = array($zip_file, $log_file);
    
        $end_time = microtime(true);
        $duration = $end_time - $start_time;
        $this->$duration = $duration;
        $message = '<p>Full Scan completed in '. $duration .' seconds</p>';
        wp_keepUp_sendMail($attachment, $message);

        // Delete all cropped files
        $files = glob( WP_KEEP_UP_RESOURCE .'quarantine/*', GLOB_BRACE);
        foreach($files as $file){ // iterate files
            if(is_file($file)) {
                unlink($file); // delete file
            }
        }
  
        echo <<<HTML
            <div class="notice notice-success is-dismissible">
                $message
            </div>
        HTML;

    }
}

public function wp_keep_update( $file, $source, $backup = '', $quarantine_dir ) { // passing directory into function for checking
    $log_file = WP_KEEP_UP_RESOURCE .'log.csv';
    $log_data = array(date('Y-m-d'), date('H:i:s'), 'Full Scan', strlen(file_get_contents($file)));

    $file_content = file_get_contents($file); // check for file content
    $source_content = file_get_contents($source); // with legit

    if($file_content != $source_content) { // comparision of files
        
        // Copy file in quarantine directory
        if(!is_dir(dirname($quarantine_dir))) {
            mkdir(dirname($quarantine_dir), 0777, true);
        } 
        copy($file, $quarantine_dir); // move from infected to quaruntine
        
        $quarantine_len = strlen($file_content);
        $source_len = strlen($source_content);
        

        if($backup != '') {     // also check if there are predefined files in backup folder          
            $backup_content = file_get_contents($backup);
            file_put_contents($backup, $backup_content);
            // if exists then use from backup rather from downloaded copy. 
            clearstatcache();
            $file_permission = substr(sprintf('%o', fileperms($file)), -4);
            //logging action whihc are done previously. 
            array_push($log_data, $quarantine_len, $source_len, basename($file), $quarantine_dir, $file_permission, 'Replaced from backup');
            $file = fopen($log_file, 'a');
            fputcsv($file, $log_data);
            fclose($file);
        } else {
            file_put_contents($file, $source_content);

            clearstatcache();
            $file_permission = substr(sprintf('%o', fileperms($file)), -4);
            array_push($log_data, $source_len, basename($file), $quarantine_dir, $file_permission, 'Replaced from Source');
            $file = fopen($log_file, 'a');
            fputcsv($file, $log_data);
            fclose($file);
        }
    }
}

    /**
     * Download WordPress with the current website version
     */
    public function download_wordpress() {
        # wordpress version
        global $wp_version;
        $file_name = 'wordpress-' .$wp_version;

        $url = "https://wordpress.org/{$file_name}.zip";
        $file_path = WP_KEEP_UP_RESOURCE . $file_name .'.zip';
        $file = fopen($file_path, "w");
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $url);
        curl_setopt($ch, CURLOPT_FILE, $file);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
        curl_exec($ch);
        curl_close($ch);
        fclose($file);
        $zip = new ZipArchive;
        $res = $zip->open($file_path);
        if ($res === TRUE) {
            $zip->extractTo(WP_KEEP_UP_RESOURCE);
            $zip->close();
            unlink($file_path);
        }
    }

    public function dir_is_empty($dir) {
        $handle = opendir($dir);
        while (false !== ($entry = readdir($handle))) {
          if ($entry != "." && $entry != "..") {
            closedir($handle);
            return false;
          }
        }
        closedir($handle);
        return true;
      }
}