<?php
/**
 * Predefined files secure by specific keywords and characters.
 */
class WP_Keep_Up_Quick_Scan {

    /**
     * 
     */
    public function __construct() {

        /**
         * Quick scan roles
         * 
         * Files and condition of file scan role
         * 
         * @var array $roles 'wp_keep_up_security_roles' filter hook returning roles array
         */
        $roles = apply_filters('wp_keep_up_security_roles', array());
     
        
        $start_time = microtime(true);
        // using the file rules.pgp based on core structure of file or content(theme structure) 
        foreach($roles as $role) {
            if($role['file_type'] == 'core') {

                foreach($role['files'] as $file) {

                    $this->scanCoreFile($file, $role['characters'], $role['words']);
                }

            } elseif( $role['file_type'] = 'content' ) {
                
                foreach($role['files'] as $file) {
                    $this->scanContentFile( $file, $role['characters'], $role['words'] );
                }   
            }
        }
      
        
        $log_file = WP_KEEP_UP_RESOURCE .'log.csv';
        $attachment = array(quarantine_resource_zip(), $log_file);
        $end_time = microtime(true);
        $duration = $end_time - $start_time;
        $message = '<p>Quick Scan completed in '. $duration .' seconds</p>';
        // wp_keepUp_sendMail($attachment, $message);
        
        echo <<<HTML
            <div class="notice notice-success is-dismissible">
                $message
            </div>
        HTML;
    }



    /**
     * Scan WordPress Core Files
     * 
     * @param string $file Relative file path from WordPress root
     * @param array $characters Character list of condition to unify affected file
     * @param array $words Word list of condition to unify affected file
     * 
     * @return boolean
     */
    private function scanCoreFile($file, $characters, $words)
    {
        // get log file location
        $log_file = WP_KEEP_UP_RESOURCE .'log.csv';

        // Collect log data
        $log_data = array(date('Y-m-d'), date('H:i:s'), 'Quick Scan');

        // Targeted file relative path
        $full_path = ABSPATH . $file;

        if(file_exists($full_path)) {

            if($this->isSecureByCharacters($full_path, $characters ) && $this->isSecureByWords($full_path, $words)) {
                // check and replace if thare are issues in scanned file
            } else {
                $this->secureTheCoreFile( $full_path );
            }

        } else { /// loggiong mechanisum for quick scan
            $log_data = array(date('Y-m-d'), date('H:i:s'), 'Quick Scan', '', '', $file, 'File Not Found', '', '', 'Root');
            $file = fopen($log_file, 'a');
            fputcsv($file, $log_data);
            fclose($file);
        }
    }


    /**
     * Check the file security validation by the character limitation
     * 
     * @param string $file_path The file absolute path
     * @param array $characters Character list with the character count condition
     * 
     * @return boolean
     */
    public function isSecureByCharacters($file_path, $characters)
    {

        $file_content = file_get_contents($file_path);
        $file_content = strtolower($file_content);

        // Checking the file is't empty
        if(strlen($file_content) < 1) return true;

        // Security check by characters
        foreach( $characters as $character => $limitation) {
            
            $repeated_length = $this->getCharacterRepetitionLength($file_content, $character);

           
    
            if($limitation <= $repeated_length) {
   
                $suspicious_character[$character] = $repeated_length; 
            }
        }

        if( isset($suspicious_character) ) {
            return false;
        }
        return true;
    }


    /**
     * Count repeated characters in the string
     * 
     * @param string $string The string
     * @param string $character the character to count repetition
     * 
     * @return int repeated length
     */
    public function getCharacterRepetitionLength($string, $character)
    {
        // Check the string and the character validatory
        if(strlen($string) < 1 || strlen($character) < 1) return;

        $string_characters_array = str_split($string);
        $repeated_character = array();
        foreach($string_characters_array as $char) {

            if( strtolower($char) == strtolower($character) ) {

                $repeated_character[] = $char;
            }
        }
        return count($repeated_character);
    }


    /**
     * Check the file security validation by the word limitation
     * 
     * @param string $file_path The file absolute path
     * @param array $words Word list with the word count condition
     * 
     * @return boolean
     */
    public function isSecureByWords($file_path, $words)
    {
        $file_content = file_get_contents($file_path);
        $file_content = strtolower($file_content);

        // Checking the file is't empty
        if(strlen($file_content) < 1) return true;

        // Security check by keywords
        foreach( $words as $word => $limitation) {

            // regular expression formate
            $pattern = '/' . $word . '/i';
            if( !preg_match_all($pattern, $file_content, $matches) ) continue;
            
            $repeated_length = count($matches[0]);
           

            if($limitation <= $repeated_length) {
                $suspicious_words[$word] = $repeated_length; 
            }
        }
        
       
        if( isset($suspicious_words) ) {
            return false;
        }
        return true;
    }


    /**
     * Scan WordPress Core Files
     * 
     * @param string $file Relative file path from WordPress root
     * @param array $characters Character list of condition to unify affected file
     * @param array $words Word list of condition to unify affected file
     * 
     * @return boolean
     */
    private function scanContentFile($file, $characters, $words)
    {
        // get log file location
        $log_file = WP_KEEP_UP_RESOURCE .'log.csv';

        // Targeted file relative path
        $full_path = ABSPATH . $file;

        
        if(file_exists($full_path)) {

            
            if($this->isSecureByCharacters($full_path, $characters ) && $this->isSecureByWords($full_path, $words)) {

            } else {
              
                $this->secureTheContentFile( $full_path );
            }

        } else {
            $log_data = array(date('Y-m-d'), date('H:i:s'), '', '', $file, 'File Not Found', '', '', 'Theme');
            $file = fopen($log_file, 'a');
            fputcsv($file, $log_data);
            fclose($file);
        }
    }
    

     /**
     * Secure WordPress Suspicious File
     * 
     * Replace suspicious file by backup file
     * 
     * @param string $file_path The file absolute path
     * @return void
     */
    private function secureTheCoreFile($file_path)
    {
      
        // log file location
        $log_file = WP_KEEP_UP_RESOURCE .'log.csv';

        // Log data collection
        $log_data = array(date('Y-m-d'), date('H:i:s'), 'Quick Scan', strlen(file_get_contents($file_path)));

        $relative_path = substr($file_path, strlen(ABSPATH));
        $quarantine_dir = WP_KEEP_UP_RESOURCE .'quarantine/' .$relative_path;
        $fresh_file = WP_KEEP_UP_RESOURCE . 'wordpress/' . $relative_path;
        
        if( !is_dir(dirname($quarantine_dir)) ) {
            mkdir(dirname($quarantine_dir), 0777, true);
        }
        copy($file_path, $quarantine_dir);
       
        if(file_exists($fresh_file)) {
            $fresh_content = file_get_contents($fresh_file);
            file_put_contents($file_path, $fresh_content);

            array_push($log_data, strlen($fresh_content));
        } else {
            array_push($log_data, 'Not exist in WP');
        }

        clearstatcache();
        $file_permission = substr(sprintf('%o', fileperms($file_path)), -4);
        array_push($log_data, basename($file_path), $file_path, $file_permission, 'replaced', 'Core file');
        $file = fopen($log_file, 'a');
        fputcsv($file, $log_data);
        fclose($file);

    }


    /**
     * Secure WordPress Suspicious File
     * 
     * Replace suspicious file by backup file
     * 
     * @param string $file_path The file absolute path
     * @return void
     */
    private function secureTheContentFile($file_path)
    {
        
        // log file location
        $log_file = WP_KEEP_UP_RESOURCE .'log.csv';

        // Log data collection
        $log_data = array(date('Y-m-d'), date('H:i:s'), 'Quick Scan', strlen(file_get_contents($file_path)));


      
        $relative_path = substr($file_path, strlen(ABSPATH));
        $quarantine_dir = WP_KEEP_UP_RESOURCE .'quarantine/' .$relative_path;
        $backup_file = WP_KEEP_UP_RESOURCE . 'backup_file/' . $relative_path;
        

      

        if( !is_dir(dirname($quarantine_dir)) ) {

            mkdir(dirname($quarantine_dir), 0777, true);
        }
        copy($file_path, $quarantine_dir);

        if(file_exists($backup_file)) {
            $fresh_content = file_get_contents($backup_file);
            file_put_contents($file_path, $fresh_content);

            array_push($log_data, strlen($fresh_content));

        } else {
            array_push($log_data, 'Not exist in Backup');
        }

        clearstatcache();
        $file_permission = substr(sprintf('%o', fileperms($file_path)), -4);
        array_push($log_data, basename($file_path), $file_path, $file_permission, 'replaced', 'Content file');
        $file = fopen($log_file, 'a');
        fputcsv($file, $log_data);
        fclose($file);

    }
}