<?php
// Function to send emails with attachments and proper configurations
function wp_keepUp_sendMail($attachments = [], $message = '') {
    // Fetch the admin email from WordPress settings
    $to = get_bloginfo('admin_email');
    
    // Subject of the email
    $subject = __('WP Keep Soul - Scan Report', 'wp-keep-up');
    
    // Current date and time for the email body
    $date = date('Y-m-d H:i:s');
    
    // Email body with HTML content
    $body = "
        <h1>WP KeepSoul Scan Report</h1>
        <p><strong>Report generated:</strong> $date</p>
        <p>$message</p>
    ";
    
    // Email headers (HTML content type)
    $headers = [
        'Content-Type: text/html; charset=UTF-8',
    ];
    
    // Try sending the email
    if (wp_mail($to, $subject, $body, $headers, $attachments)) {
        return true; // Email sent successfully
    } else {
        error_log('WP KeepSoul: Email failed to send. Check your SMTP or email server settings.');
        return false; // Email failed to send
    }
}

// Function to create a zip archive of the quarantine folder
function quarantine_resource_zip() {
    // Define the path to the quarantine folder
    $rootPath = WP_KEEP_UP_RESOURCE . 'quarantine';
    
    // Generate a unique filename for the zip file
    $date = date('Y-m-d-H-i-s');
    $file_name = "quarantine-{$date}.zip";
    $zipPath = WP_KEEP_UP_RESOURCE . $file_name;

    // Initialize ZipArchive object
    $zip = new ZipArchive();
    
    // Try opening the zip file for writing
    if ($zip->open($zipPath, ZipArchive::CREATE | ZipArchive::OVERWRITE) === true) {
        // Create recursive directory iterator for the quarantine folder
        $files = new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($rootPath, FilesystemIterator::SKIP_DOTS),
            RecursiveIteratorIterator::LEAVES_ONLY
        );
        
        // Add each file in the quarantine folder to the zip
        foreach ($files as $file) {
            $filePath = $file->getRealPath(); // Absolute file path
            $relativePath = substr($filePath, strlen($rootPath) + 1); // Relative path in zip
            
            // Add file to zip archive
            $zip->addFile($filePath, $relativePath);
        }
        
        // Close the zip archive
        $zip->close();
        
        return $file_name; // Return the name of the created zip file
    } else {
        error_log('WP KeepSoul: Failed to create zip archive.');
        return false; // Return false if zip creation fails
    }
}

// Example usage of the functions
function wp_keepUp_handleQuarantineAndSendReport() {
    // Create a zip of the quarantine folder
    $zipFileName = quarantine_resource_zip();
    if ($zipFileName) {
        $zipFilePath = WP_KEEP_UP_RESOURCE . $zipFileName; // Full path to the zip file
        
        // Ensure the file exists before attempting to send it
        if (file_exists($zipFilePath)) {
            $message = 'Attached is the latest scan report for quarantined files.';
            $attachments = [$zipFilePath];
            
            // Send the email
            $emailSent = wp_keepUp_sendMail($attachments, $message);
            
            if ($emailSent) {
                error_log('WP KeepSoul: Email sent successfully.');
            } else {
                error_log('WP KeepSoul: Email failed to send.');
            }
        } else {
            error_log("WP KeepSoul: Zip file $zipFilePath does not exist.");
        }
    } else {
        error_log('WP KeepSoul: Failed to create quarantine zip file.');
    }
}
