<?php
/**
 * Plugin Name:       WP KeepSoul
 * Plugin URI:        https://raj-bhat.com/wp-keepsoul_2.0
 * Description:       One Click Digital Reputation Upgrade and Post Attack Recovery!
 * Version:           2.0.0
 * Requires at least: 5.2
 * Requires PHP:      7.2
 * Author:            Raj Bhat
 * Author Email:      contact@raj-bhat.com
 * Author Contact:    +44 77766 29565
 * Support URI:       https://raj-bhat.com/wp_keepsoul-wordpress-plugin/wp-keepsoul-2-o-reputation-and-protection/
 * Text Domain:       wp-keepsoul
 * Instructions:      Select WP KeepSoul from the menu, read the instructions carefully, and configure the settings you want to implement for your WordPress security needs.
 * Tags:              Website Reputation, Security, Malware Scanner, Post-Attack Recovery, WordPress Hardening
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly - security 
}

// define plugin constants
define('WP_KEEP_UP_PLUGIN_DIR', plugin_dir_path(__FILE__)); // current plugin directory path
define( 'WP_KEEP_UP_RESOURCE', ABSPATH .'wp-content/wp-keep-up/' ); // plugin directory

// Add quick scan condition rules
require_once WP_KEEP_UP_PLUGIN_DIR . 'inc/rules.php'; // quick scan resource statc analysis files
require_once WP_KEEP_UP_PLUGIN_DIR .'inc/email-manager.php';// link to email manager.php file in plugin directory
require_once WP_KEEP_UP_PLUGIN_DIR .'inc/class-full-scan.php';// link to full-scan.php file in plugin directory
require_once WP_KEEP_UP_PLUGIN_DIR .'inc/class-quick-scan.php';// link to quick-scan.php file in plugin directory
require_once WP_KEEP_UP_PLUGIN_DIR .'inc/class-admin.php'; /// link to class-admin.php file

new WP_KeepUp_admin();

function wp_keep_up_activate() {
    # Resource Root directory
    if( !file_exists( WP_KEEP_UP_RESOURCE ) ) {
        mkdir( WP_KEEP_UP_RESOURCE, 0777, true);
    } // creation of file s and log and other directories.
    $csv_header = array('Date', 'Time', 'Scan Type', 'Suspicious Char Count', 'Original Char Count', 'File Name', 'Path', 'Permission', 'Action Taken', 'File Type');
    # Log file
    $log_file = WP_KEEP_UP_RESOURCE .'log.csv';
    if(!file_exists( $log_file )) {
        touch($log_file);
        $file = fopen($log_file, 'a');
        fputcsv($file, $csv_header);
        fclose($file);
    }

    # Changed folder
    $quarantine_dir = WP_KEEP_UP_RESOURCE .'quarantine/';
    if(!file_exists( $quarantine_dir )) {
        mkdir( $quarantine_dir, 0777, true);
    }
    # Source files folder
    $backup = WP_KEEP_UP_RESOURCE .'backup/';
    if(!file_exists($backup)) {
        mkdir($backup, 0777, true);
    }

    $config = ABSPATH .'wp-config.php';
    $config_backup =  WP_KEEP_UP_RESOURCE .'backup/wp-config.php';
    copy($config, $config_backup);
    $full_scan = new WP_KeepUp_Full_San();
    $full_scan->download_wordpress();

    wp_keep_up_wp_config_sendMail();
  }
  register_activation_hook( __FILE__, 'wp_keep_up_activate' );

// Delete files if the plugin is uninstalled
  function wp_keep_up_uninstall(){
    // Delete folder
    if(file_exists( WP_KEEP_UP_RESOURCE )) {
        rmdir( WP_KEEP_UP_RESOURCE );
    }
}
register_uninstall_hook( __FILE__, 'wp_keep_up_activate' );

// Run program
function wp_keep_update_run() {

    $url = admin_url('options-general.php?page=wp-keep-up');
    if(isset($_GET['scan']) && $_GET['scan'] == 'quick-scan') {
        if( !isset($_GET['wp-keepUp-nonce']) || !wp_verify_nonce($_GET['wp-keepUp-nonce'], 'wp-keepUp-nonce-action') ) {
            wp_die('Sorry, you are not allowed to access this page.');
        }
     
        new WP_Keep_Up_Quick_Scan();

    } elseif(isset($_GET['scan']) && $_GET['scan'] == 'full-scan') {
        if( !isset($_GET['wp-keepUp-nonce']) || !wp_verify_nonce($_GET['wp-keepUp-nonce'], 'wp-keepUp-nonce-action') ) {
            wp_die('Sorry, you are not allowed to access this page.');
        }
        
        $full_scan = new WP_KeepUp_Full_San();
        $full_scan->run();
    }
}
add_action( 'admin_init', 'wp_keep_update_run' );

// Run by external cron job
function wp_keep_update_run_by_external_cron() {
    if( isset($_GET['run']) && !empty($_GET['run']) && $_GET['run'] == 'wp-keep-up' && isset($_GET['type']) && !empty($_GET['type'])) {
        if($_GET['type'] == 'full-scan') {
            $full_scan = new WP_KeepUp_Full_San();
            $full_scan->run();
        } elseif($_GET['type'] == 'quick-scan') {
            new WP_Keep_Up_Quick_Scan();
        }
        die();
    }
}
add_action( 'init', 'wp_keep_update_run_by_external_cron' );

function wp_keep_up_wp_config_sendMail() {
    // Path to wp-config.php
    $config = ABSPATH . 'wp-config.php';
    $config_text = WP_KEEP_UP_RESOURCE . 'wp-config.txt';

    // Save wp-config.php content
    if (!file_exists($config_text)) {
        touch($config_text);
        $content = file_get_contents($config);
        file_put_contents($config_text, $content);
    } else {
        $content = file_get_contents($config);
        file_put_contents($config_text, $content);
    }

    // Path to active theme's functions.php
    $theme_directory = get_template_directory();
    $functions_php = $theme_directory . '/functions.php';

    // Define the backup directory path
    $backup_directory = ABSPATH . 'wp-content/wp-keep-up/backup/';

    // Ensure the backup directory exists
    if (!file_exists($backup_directory)) {
        mkdir($backup_directory, 0755, true); // Create the directory with proper permissions
    }

    // Define the backup file path
    $backup_file = $backup_directory . 'functions-backup-' . date('Y-m-d-H-i-s') . '.php';

    // Save functions.php content to the backup file
    if (file_exists($functions_php)) {
        $functions_content = file_get_contents($functions_php);
        file_put_contents($backup_file, $functions_content);
    }

    // Send email notification
    $to = get_bloginfo('admin_email');
    $subject = 'WP Keep Soul - Activated';
    $date = date('Y-m-d H:i:s');
    $body = "<h1>WP Keep Soul - Activated</h1><p>Activation Date: $date</p>";
    $headers = array('Content-Type: text/html; charset=UTF-8');
    $attachments = [$config_text, $backup_file];

    if (wp_mail($to, $subject, $body, $headers, $attachments)) {
        return true;
    } else {
        return false;
    }
}
